import scipy.io as sio
import sys
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from IPython import display
plt.rcParams.update({'font.size': 22})
plt.rcParams.update({'figure.max_open_warning': 0})

plt.interactive(True)

viscos=1/550


# makes sure figures are updated when using ipython
display.clear_output(wait=True)

datax= np.loadtxt("x2d.dat")
x=datax[0:-1]
ni=int(datax[-1])
datay= np.loadtxt("y2d.dat")
y=datay[0:-1]
nj=int(datay[-1])

x2d=np.zeros((ni+1,nj+1))
y2d=np.zeros((ni+1,nj+1))

x2d=np.reshape(x,(ni+1,nj+1))
y2d=np.reshape(y,(ni+1,nj+1))

# compute cell centers
xp2d=0.25*(x2d[0:-1,0:-1]+x2d[0:-1,1:]+x2d[1:,0:-1]+x2d[1:,1:])
yp2d=0.25*(y2d[0:-1,0:-1]+y2d[0:-1,1:]+y2d[1:,0:-1]+y2d[1:,1:])

y=yp2d[1,:]
# z grid
zmax, nk=np.loadtxt('z.dat')
nk=np.int(nk)
zp = np.linspace(0, zmax, nk)

itstep,nk,dz=np.load('itstep.npy')
p2d=np.load('p_averaged.npy')/itstep
u2d=np.load('u_averaged.npy')/itstep
v2d=np.load('v_averaged.npy')/itstep
w2d=np.load('w_averaged.npy')/itstep
k_model_2d=np.load('k_averaged.npy')/itstep
vis2d=np.load('vis_averaged.npy')/itstep
uu2d=np.load('uu_stress.npy')/itstep
vv2d=np.load('vv_stress.npy')/itstep
ww2d=np.load('ww_stress.npy')/itstep
uv2d=np.load('uv_stress.npy')/itstep
fk2d=np.load('fk_averaged.npy')/itstep
om2d=np.load('om_averaged.npy')/itstep
eps2d=np.load('eps_averaged.npy')/itstep

uu2d=uu2d-u2d**2
uv2d=uv2d-u2d*v2d

k2d=0.5*(uu2d+vv2d+ww2d)


u=np.mean(u2d,axis=0)
uu=np.mean(uu2d,axis=0)
vv=np.mean(vv2d,axis=0)
ww=np.mean(ww2d,axis=0)
uv=np.mean(uv2d,axis=0)
k=np.mean(k2d,axis=0)
k_model=np.mean(k_model_2d,axis=0)
fk=np.mean(fk2d,axis=0)
om=np.mean(om2d,axis=0)
eps=np.mean(eps2d,axis=0)
vis=np.mean(vis2d,axis=0)

vist=vis-viscos
dudy=np.gradient(u,y)
uv_model=-vist*dudy



ustar=(viscos*u[1]/y[1])**0.5

yplus=y*ustar/viscos


f_e_mean=np.load('f_e_mean.npy')/itstep
f_d_mean=np.load('f_d_mean.npy')/itstep
l_c_mean=np.load('l_c_mean.npy')/itstep
l_tilde_mean=np.load('l_tilde_mean.npy')/itstep
f_dt_mean=np.load('f_dt_mean.npy')/itstep
f_b_mean=np.load('f_b_mean.npy')/itstep
f_e1_mean=np.load('f_e1_mean.npy')/itstep
f_e2_mean=np.load('f_e2_mean.npy')/itstep
denom_mean=np.load('denom_mean.npy')/itstep

r_dt=(vis-1)*viscos/denom_mean
r_dl=viscos/denom_mean

l_u=k_model**1.5/eps


#      y/h             y+              U+             u'+             v'+             w'+           -Om_z+          om_x'+           om_y'+           om_z'+         uv'+             uw'+           vw'+             pr'+            ps'+          psto'+            p'
# -----------------------------------------------------
DNS_mean=np.genfromtxt("/chalmers/users/lada/DNS_channel_550_950_2000/Re550.dat",comments="%")
y_DNS=DNS_mean[:,0];
yplus_DNS=DNS_mean[:,1];
u_DNS=DNS_mean[:,2];
u2_DNS=DNS_mean[:,3]**2;
v2_DNS=DNS_mean[:,4]**2;
w2_DNS=DNS_mean[:,5]**2;
uv_DNS=DNS_mean[:,10];



k_DNS=0.5*(u2_DNS+v2_DNS+w2_DNS)

lst=list(range(len(u_DNS)))
iDNS=lst[0::45]

# find equi.distant DNS cells in log-scale
xx=0.
jDNS=[1]*40
for i in range (0,40):
   i1 = (np.abs(10.**xx-yplus_DNS)).argmin()
   jDNS[i]=int(i1)
   xx=xx+0.2


########################################## damping functions
fig59,ax1 = plt.subplots()
plt.subplots_adjust(left=0.20,bottom=0.20)
plt.plot(f_b_mean,yplus,'b',label="$f_b$")
plt.plot(f_e_mean,yplus,'r-',label="$f_e$")
plt.plot(f_e1_mean,yplus,'r--',label="$f_{e1}$")
plt.plot(f_e2_mean,yplus,'k-',label="$f_{e2}$")
plt.legend(loc="upper right",prop=dict(size=18))
plt.ylabel(r"$y^+$")
plt.axis([0,2.2,0,200])
plt.savefig('f_b-f_e-damping-functions.png')

########################################## damping functions
fig59,ax1 = plt.subplots()
plt.subplots_adjust(left=0.20,bottom=0.20)
i1=200
plt.plot(f_d_mean,yplus,'k-',label="$f_d$")
plt.plot(r_dl,yplus,'b--',label="$r_{dl}$")
plt.plot(r_dt,yplus,'r-',label="$r_{dt}$")
plt.legend(loc="upper right",prop=dict(size=18))
plt.ylabel(r"$y^+$")
plt.axis([0,2.2,0,550])
plt.savefig('f_d-r_dt-r_dl-damping-functions.png')



########################################## U 
fig1,ax1 = plt.subplots()
plt.subplots_adjust(left=0.20,bottom=0.20)
i1 = 0
plt.semilogx(yplus,u,'b-')
plt.semilogx(yplus_DNS[jDNS],u_DNS[jDNS],'bo')
plt.ylabel("$U^+$")
plt.xlabel("$y^+$")
plt.axis([1, 550, 0, 28])
plt.savefig('u_log_550-python.png',bbox_inches='tight')


########################################## uu 
fig1,ax1 = plt.subplots()
plt.subplots_adjust(left=0.20,bottom=0.20)
i1 = 0
plt.plot(yplus,uu,'b-')
plt.plot(yplus_DNS[jDNS],u2_DNS[jDNS],'bo')
plt.ylabel(r"$\overline{u'u'}$")
plt.xlabel("$y^+$")
plt.axis([1, 550, 0, 11])
plt.savefig('uu_550-python.png',bbox_inches='tight')



########################################## vv 
fig1,ax1 = plt.subplots()
plt.subplots_adjust(left=0.20,bottom=0.20)
i1 = 0
plt.plot(yplus,vv,'b-')
plt.plot(yplus_DNS[iDNS],v2_DNS[iDNS],'bo')
plt.ylabel(r"$\overline{v'v'}$")
plt.xlabel("$y^+$")
plt.axis([1, 550, 0, 2])
plt.savefig('vv_550-python.png',bbox_inches='tight')



########################################## ww 
fig1,ax1 = plt.subplots()
plt.subplots_adjust(left=0.20,bottom=0.20)
i1 = 0
plt.plot(yplus,ww,'b-')
plt.plot(yplus_DNS[iDNS],w2_DNS[iDNS],'bo')
plt.ylabel(r"$\overline{w'w'}$")
plt.xlabel("$y^+$")
plt.axis([1, 550, 0, 2])
plt.savefig('ww_550-python.png',bbox_inches='tight')



########################################## uv 
fig1,ax1 = plt.subplots()
plt.subplots_adjust(left=0.20,bottom=0.20)
i1 = 0
plt.plot(yplus,uv_model,'r--')
plt.plot(yplus,uv,'b-')
plt.plot(yplus_DNS[iDNS],uv_DNS[iDNS],'bo')
plt.ylabel(r"$\overline{u'v'}$")
plt.xlabel("$y^+$")
plt.axis([1, 550, -1, 0])
plt.savefig('uv_550-python.png',bbox_inches='tight')


########################################## k 
fig1,ax1 = plt.subplots()
plt.subplots_adjust(left=0.20,bottom=0.20)
i1 = 0
plt.plot(yplus,k,'b-')
plt.plot(yplus_DNS[iDNS],k_DNS[iDNS],'bo')
plt.ylabel(r"$k^+$")
plt.xlabel("$y^+$")
plt.axis([1, 550, 0, 5])
plt.savefig('k_550-python.png',bbox_inches='tight')

########################################## vis 
fig1,ax1 = plt.subplots()
plt.subplots_adjust(left=0.20,bottom=0.20)
i1 = 0
plt.plot(yplus,vis/viscos,'b-')
plt.ylabel(r"$\nu_t/\nu$")
plt.xlabel("$y^+$")
plt.axis([1, 550, 0, 10])
plt.savefig('vis_550-python.png',bbox_inches='tight')

########################################## k 
fig1,ax1 = plt.subplots()
plt.subplots_adjust(left=0.20,bottom=0.20)
i1 = 0
plt.plot(yplus,fk,'b-')
plt.ylabel(r"$f_k$")
plt.xlabel("$y^+$")
plt.axis([1, 550, 0.7, 2])
plt.savefig('fk_550-python.png',bbox_inches='tight')

########################################## k 
fig1,ax1 = plt.subplots()
plt.subplots_adjust(left=0.20,bottom=0.20)
i1 = 0
plt.plot(yplus,fk,'b-')
plt.ylabel(r"$f_k$")
plt.xlabel("$y^+$")
plt.axis([1, 250, 0.7, 2])
plt.savefig('fk_550-zoom.png',bbox_inches='tight')

########################################## l_c l_u 
fig1,ax1 = plt.subplots()
plt.subplots_adjust(left=0.20,bottom=0.20)
i1 = 0
plt.plot(yplus,l_u,'b-',label='$l_u$')
plt.plot(yplus,l_c_mean,'r--',label='$l_c$')
plt.ylabel(r"$l_u \quad l_c$")
plt.xlabel("$y^+$")
plt.legend(loc="upper right",prop=dict(size=18))
plt.axis([1, 250, 0,0.2])
plt.savefig('lc-lu-python.png',bbox_inches='tight')
