import scipy.io as sio
import sys
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
import matplotlib.ticker as mtick
from matplotlib import ticker

plt.rcParams.update({'font.size': 22})

plt.interactive(True)
plt.close('all')

viscos=1/10000

datax= np.loadtxt("x2d.dat")
x=datax[0:-1]
ni=int(datax[-1])
datay= np.loadtxt("y2d.dat")
y=datay[0:-1]
nj=int(datay[-1])

x2d=np.zeros((ni+1,nj+1))
y2d=np.zeros((ni+1,nj+1))

x2d=np.reshape(x,(ni+1,nj+1))
y2d=np.reshape(y,(ni+1,nj+1))

# compute cell centers
xp2d=0.25*(x2d[0:-1,0:-1]+x2d[0:-1,1:]+x2d[1:,0:-1]+x2d[1:,1:])
yp2d=0.25*(y2d[0:-1,0:-1]+y2d[0:-1,1:]+y2d[1:,0:-1]+y2d[1:,1:])

x=xp2d[:,0]
y=yp2d[0,:]


u2d=np.load('u2d_saved.npy')
p2d=np.load('p2d_saved.npy')
v2d=np.load('v2d_saved.npy')
k2d=np.load('k2d_saved.npy')
om2d=np.load('om2d_saved.npy')
vis2d=np.load('vis2d_saved.npy')

dudx2d,dudy2d=np.gradient(u2d,x,y)
uv2d=-vis2d*dudy2d

tauw=viscos*u2d[0,0]/yp2d[0,0]


# compute cf
cf=np.zeros(ni)
ustarv=np.zeros(ni)
for i in range (0,ni-1):
     xt=x2d[i,0]-x2d[i-1,0]
     yt=y2d[i,0]-y2d[i-1,0]
     rl=(xt**2+yt**2)**0.5
     xt=xt/rl
     yt=yt/rl
     uwall=abs(u2d[i,0]*xt+v2d[i,0]*yt)
     xn=-yt
     yn=xt
     yp=0.5*abs((y2d[i,1]-y2d[i,0])*yn+(x2d[i,1]-x2d[i,1])*xn)
     ustar=(uwall*viscos/yp)**0.5
     ustars=(uwall*viscos/yp)**0.5*u2d[i,0]/abs(u2d[i,0])
     cf[i]=abs(ustars)*ustars/0.5

ustarv[0]=ustarv[1]

cf[0]=cf[1]


#load force history
force_flow= np.loadtxt("force-history.dat")
force=force_flow[:,0]
flow=force_flow[:,1]


#load iteration history
u_iter= np.loadtxt("u-iter.dat")

u2=u_iter[:,0]
u5=u_iter[:,1]
u10=u_iter[:,2]
u20=u_iter[:,3]

########################################## Cf 
fig16,ax1 = plt.subplots()
plt.subplots_adjust(left=0.25,bottom=0.20)
plt.plot(xp2d[:,1],cf,'b-',2)

plt.xlabel(r"$x$")
plt.ylabel("$C_f$")
plt.axis([0,3,-0.02,0.03])
plt.savefig('cf.png')


########################################## U 
fig1,ax1 = plt.subplots()
plt.subplots_adjust(left=0.20,bottom=0.20)
xx=0
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(u2d[i1,:],yp2d[i1,:]-y2d[i1,0],'b-',label="$x=0$")
xx=0.5
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(u2d[i1,:],yp2d[i1,:]-y2d[i1,0],'r--',label="$x=0.5$")
xx=1.0
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(u2d[i1,:],yp2d[i1,:]-y2d[i1,0],'k-.',label="$x=1.0$")
xx=1.5
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(u2d[i1,:],yp2d[i1,:]-y2d[i1,0],'b--',label="$x=1.5$")
xx=2.0
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(u2d[i1,:],yp2d[i1,:]-y2d[i1,0],'k--',label="$x=2.0$")
xx=2.5
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(u2d[i1,:],yp2d[i1,:]-y2d[i1,0],'r-',label="$x=2.5$")

plt.legend(loc='best',prop=dict(size=13))
plt.xlabel("$U$")
plt.ylabel("$y$")
plt.axis([-0.2, 1.3, 0, 1.0])
plt.savefig('u_wavy-channel.png',bbox_inches='tight')

########################################## uv 
fig1,ax1 = plt.subplots()
plt.subplots_adjust(left=0.25,bottom=0.20)
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(uv2d[i1,:],yp2d[i1,:]-y2d[i1,0],'b-',label="$x=0$")
xx=0.5
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(uv2d[i1,:],yp2d[i1,:]-y2d[i1,0],'r--',label="$x=0.5$")
xx=1.0
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(uv2d[i1,:],yp2d[i1,:]-y2d[i1,0],'k-.',label="$x=1.0$")
xx=1.5
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(uv2d[i1,:],yp2d[i1,:]-y2d[i1,0],'b--',label="$x=1.5$")
xx=2.0
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(uv2d[i1,:],yp2d[i1,:]-y2d[i1,0],'k--',label="$x=2.0$")
xx=2.5
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(uv2d[i1,:],yp2d[i1,:]-y2d[i1,0],'r-',label="$x=2.5$")

plt.legend(loc='best',prop=dict(size=13))
plt.xlabel("$\overline{u'v'}$")
M = 3 # number of large ticks
xticks = ticker.MaxNLocator(M)
ax1.xaxis.set_major_locator(xticks)

plt.ylabel("$y$")
plt.axis([-0.01, 0.002, 0, 1])
plt.savefig('uv_wavy-channel.png',bbox_inches='tight')

########################################## vis 
fig1,ax1 = plt.subplots()
plt.subplots_adjust(left=0.25,bottom=0.20)
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(vis2d[i1,:]/viscos,yp2d[i1,:]-y2d[i1,0],'b-',label="$x=0$")
xx=0.5
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(vis2d[i1,:]/viscos,yp2d[i1,:]-y2d[i1,0],'r--',label="$x=0.5$")
xx=1.0
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(vis2d[i1,:]/viscos,yp2d[i1,:]-y2d[i1,0],'k-.',label="$x=1.0$")
xx=1.5
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(vis2d[i1,:]/viscos,yp2d[i1,:]-y2d[i1,0],'b--',label="$x=1.5$")
xx=2.0
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(vis2d[i1,:]/viscos,yp2d[i1,:]-y2d[i1,0],'k--',label="$x=2.0$")
xx=2.5
i1 = (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(vis2d[i1,:]/viscos,yp2d[i1,:]-y2d[i1,0],'r-',label="$x=2.5$")

plt.legend(loc='best',prop=dict(size=13))
plt.xlabel(r"$\nu_t/\nu$")
M = 3 # number of large ticks
xticks = ticker.MaxNLocator(M)
ax1.xaxis.set_major_locator(xticks)

plt.ylabel("$y$")
plt.axis([0, 100, 0, 1])
plt.savefig('vis_wavy-channel.png',bbox_inches='tight')


########################################## u iteration
fig1,ax1 = plt.subplots()
plt.subplots_adjust(left=0.20,bottom=0.20)
plt.plot(u2,'b-',label="$j=3$")
plt.plot(u5,'r-',label="$j=11$")
plt.plot(u10,'b--',label="$j=22$")
plt.plot(u20,'k-',label="$j=28$")
plt.legend(loc='best',prop=dict(size=13))
plt.xlabel('iteration')
plt.ylabel('$U$')
plt.savefig('u_vs_iteration.png',bbox_inches='tight')

########################################## force
fig1,ax1 = plt.subplots()
plt.subplots_adjust(left=0.20,bottom=0.20)
plt.plot(force,'b-',label="$j=3$")
plt.xlabel('iteration')
plt.ylabel('force')
plt.savefig('force_vs_iteration.png',bbox_inches='tight')

########################################## flow`
fig1,ax1 = plt.subplots()
plt.subplots_adjust(left=0.20,bottom=0.20)
plt.plot(flow,'b-',label="$j=3$")
plt.xlabel('iteration')
plt.ylabel('force')
plt.savefig('flow_vs_iteration.png',bbox_inches='tight')

