import scipy.io as sio
import sys
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1.inset_locator import inset_axes

plt.interactive(True)

plt.close('all')




# read data file
tec=np.genfromtxt("tec.dat", dtype=None,comments="%")

#text='VARIABLES = X Y P U V u2 v2 w2 uv mu_sgs prod'

x=tec[:,0]
y=tec[:,1]
p=tec[:,2]
u=tec[:,3]
v=tec[:,4]
uu=tec[:,5]
vv=tec[:,6]
ww=tec[:,7]
uv=tec[:,8]
k=0.5*(uu+vv+ww)

if max(y) == 1.:
   ni=170
   nj=194
   nu=1./10000.
else:
   nu=1./10595.
   if max(x) > 8.:
     nj=162
     ni=162
   else:
     ni=402
     nj=162

viscos=nu

u2d_DNS=np.reshape(u,(nj,ni))
v2d_DNS=np.reshape(v,(nj,ni))
p2d_DNS=np.reshape(p,(nj,ni))
x2d_DNS=np.reshape(x,(nj,ni))
y2d_DNS=np.reshape(y,(nj,ni))
uu2d_DNS=np.reshape(uu,(nj,ni)) #=mean{v'_1v'_1}
uv2d_DNS=np.reshape(uv,(nj,ni)) #=mean{v'_1v'_2}
vv2d_DNS=np.reshape(vv,(nj,ni)) #=mean{v'_2v'_2}
k2d_DNS=np.reshape(k,(nj,ni))

u2d_DNS=np.transpose(u2d_DNS)
v2d_DNS=np.transpose(v2d_DNS)
p2d_DNS=np.transpose(p2d_DNS)
xp2d_DNS=np.transpose(x2d_DNS)
yp2d_DNS=np.transpose(y2d_DNS)
uu2d_DNS=np.transpose(uu2d_DNS)
vv2d_DNS=np.transpose(vv2d_DNS)
uv2d_DNS=np.transpose(uv2d_DNS)

# fix Neumann at the top boundary
u2d_DNS[:,-1]=u2d_DNS[:,-2]


# load RANS predictions (with pyCALC-RANS)
datax= np.loadtxt("x2d.dat")
x=datax[0:-1]
ni=int(datax[-1])
datay= np.loadtxt("y2d.dat")
y=datay[0:-1]
nj=int(datay[-1])

x2d=np.zeros((ni+1,nj+1))
y2d=np.zeros((ni+1,nj+1))

x2d=np.reshape(x,(ni+1,nj+1))
y2d=np.reshape(y,(ni+1,nj+1))

# compute cell centers
xp2d=0.25*(x2d[0:-1,0:-1]+x2d[0:-1,1:]+x2d[1:,0:-1]+x2d[1:,1:])
yp2d=0.25*(y2d[0:-1,0:-1]+y2d[0:-1,1:]+y2d[1:,0:-1]+y2d[1:,1:])

p2d=np.load('p2d_saved.npy')
u2d=np.load('u2d_saved.npy')
k2d=np.load('k2d_saved.npy')
vis2d=np.load('vis2d_saved.npy')
om2d=np.load('om2d_saved.npy')
k_model2d=np.load('k2d_saved.npy')


#************
# plot U at x=0.5
fig3 = plt.figure("Figure 3")
xx=0.5
iDNS = (np.abs(xx-xp2d_DNS[:,1])).argmin()  # find index which closest fits xx
plt.plot(u2d_DNS[iDNS,:],yp2d_DNS[iDNS,:],'r-')
i= (np.abs(xx-xp2d[:,1])).argmin()  # find index which closest fits xx
plt.plot(u2d[i,:],yp2d[i,:],'b-')
plt.xlabel('$U')
plt.ylabel('y')
plt.savefig('u_wave.png')

